package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.entity.LisConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.LisConfigService;
import com.diagbot.service.impl.LisConfigServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/7/29 15:03
 */
@Component
public class LisConfigFacade extends LisConfigServiceImpl {
    @Autowired
    @Qualifier("lisConfigServiceImpl")
    private LisConfigService lisConfigService;

    /**
     * 判断是否已存在
     *
     * @param lisConfig
     * @return
     */
    public Boolean isExistRecord(LisConfig lisConfig) {
        QueryWrapper<LisConfig> queryWrapper = new QueryWrapper<>();
        LisConfig oldRecord = new LisConfig();
        if (lisConfig.getId() != null) {
            oldRecord = this.getById(lisConfig.getId());
            if (oldRecord != null && oldRecord.getIsDeleted().equals(IsDeleteEnum.N.getKey())) {
                return true;
            }
        } else if (StringUtil.isNotBlank(lisConfig.getHisName())) {
            queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("his_name", lisConfig.getHisName());
            if (StringUtil.isBlank(lisConfig.getHisDetailName())) {
                queryWrapper
                        .isNull("his_detail_name")
                        .or()
                        .eq("his_detail_name", "");
            } else {
                queryWrapper
                        .eq("his_detail_name", lisConfig.getHisDetailName());
            }
            oldRecord = this.getOne(queryWrapper);
            if (oldRecord != null) {
                return true;
            }
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param lisConfig
     * @return
     */
    public LisConfig saveRecord(LisConfig lisConfig) {
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        if (lisConfig.getId() != null) {
            deleteRecord(lisConfig.getId());
            lisConfig.setId(null);
        }
        lisConfig.setCreator(userId);
        lisConfig.setGmtCreate(now);
        lisConfig.setModifier(userId);
        lisConfig.setGmtModified(now);
        this.saveOrUpdate(lisConfig);
        return lisConfig;
    }

    /**
     * 保存记录-批量
     *
     * @param lisConfigList
     * @return
     */
    public Boolean saveRecords(List<LisConfig> lisConfigList) {
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        List<Long> deleteIds = Lists.newLinkedList();
        if (ListUtil.isNotEmpty(lisConfigList)) {
            lisConfigList.forEach(lisConfig -> {
                if (lisConfig.getId() != null) {
                    deleteIds.add(lisConfig.getId());
                    lisConfig.setId(null);
                    lisConfig.setCreator(userId);
                    lisConfig.setGmtCreate(now);
                    lisConfig.setModifier(userId);
                    lisConfig.setGmtModified(now);
                }
            });
        }
        deleteRecords(deleteIds);
        lisConfigService.saveBatch(lisConfigList);
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param id
     * @return
     */
    public Boolean deleteRecord(Long id) {
        UpdateWrapper<LisConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id)
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        this.update(updateWrapper);
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param ids
     * @return
     */
    public Boolean deleteRecords(List<Long> ids) {
        if (ListUtil.isEmpty(ids)) {
            return false;
        }
        UpdateWrapper<LisConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", ids)
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        this.update(updateWrapper);
        return true;
    }

    /**
     * 化验数据导入
     *
     * @param file
     */
    public void importExcel(MultipartFile file) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        List<LisConfig> lisConfigList = ExcelUtils.importExcel(file, 0, 1, LisConfig.class);
        if (ListUtil.isNotEmpty(lisConfigList)) {
            //过滤套餐或公表名为空的数据
            lisConfigList = lisConfigList
                    .stream()
                    .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                    .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                    .collect(Collectors.toList());
            //验证数据是否已存在

            //新增数据
            lisConfigList.forEach(lisConfig -> {
                lisConfig.setIsDeleted(IsDeleteEnum.N.getKey());
                lisConfig.setHospitalId(Long.valueOf(hospitalId));
                lisConfig.setCreator(userId);
                lisConfig.setGmtCreate(now);
                lisConfig.setModifier(userId);
                lisConfig.setGmtModified(now);
            });

            lisConfigService.saveBatch(lisConfigList);
        }
    }
}